migrate_vma->cpages originally contained a count of the number of
pages migrating including non-present pages which can be poluated
directly on the target.

Commit 241f68859656 ("mm/migrate_device.c: refactor migrate_vma and
migrate_deivce_coherent_page()") inadvertantly changed this to contain
just the number of pages that were unmapped. Usage of
migrate_vma->cpages isn't documented, but most drivers use it to see
if all the requested addresses can be migrated so restore the original
behaviour.

Fixes: 241f68859656 ("mm/migrate_device.c: refactor migrate_vma and 
migrate_deivce_coherent_page()")
Signed-off-by: Alistair Popple <apop...@nvidia.com>
Reported-by: Ralph Campbell <rcampb...@nvidia.com>
Cc: John Hubbard <jhubb...@nvidia.com>
Cc: Alex Sierra <alex.sie...@amd.com>
Cc: Ben Skeggs <bske...@redhat.com>
Cc: Felix Kuehling <felix.kuehl...@amd.com>
Cc: Lyude Paul <ly...@redhat.com>
Cc: Jason Gunthorpe <j...@nvidia.com>
Cc: Michael Ellerman <m...@ellerman.id.au>
---

Hi Andrew, hoping you can merge this small fix which Ralph reported to
me for v6.1-rcX.

---
 mm/migrate_device.c | 8 ++++++--
 1 file changed, 6 insertions(+), 2 deletions(-)

diff --git a/mm/migrate_device.c b/mm/migrate_device.c
index 6fa682eef7a0..721b2365dbca 100644
--- a/mm/migrate_device.c
+++ b/mm/migrate_device.c
@@ -357,7 +357,8 @@ static bool migrate_vma_check_page(struct page *page, 
struct page *fault_page)
 }
 
 /*
  * Unmaps pages for migration. Returns number of unmapped pages.
  */
 static unsigned long migrate_device_unmap(unsigned long *src_pfns,
                                          unsigned long npages,
@@ -373,8 +374,11 @@ static unsigned long migrate_device_unmap(unsigned long 
*src_pfns,
                struct page *page = migrate_pfn_to_page(src_pfns[i]);
                struct folio *folio;
 
-               if (!page)
+               if (!page) {
+                       if (src_pfns[i] & MIGRATE_PFN_MIGRATE)
+                               unmapped++;
                        continue;
+               }
 
                /* ZONE_DEVICE pages are not on LRU */
                if (!is_zone_device_page(page)) {
@@ -357,7 +357,8 @@ static bool migrate_vma_check_page(struct page *page, 
 
 /*
- * Unmaps pages for migration. Returns number of unmapped pages.
+ * Unmaps pages for migration. Returns number of source pfns marked as
+ * migrating.
  */
 static unsigned long migrate_device_unmap(unsigned long *src_pfns,
-- 
2.35.1

Reply via email to